@@ -22,21 +22,23 @@ namespace XSDDiagram
 	{
 		private static XmlSerializer schemaSerializer = new XmlSerializer(typeof(XMLSchema.schema));
 		private Diagram diagram = new Diagram();
-		//private Hashtable hashtableNamespaceUrlByPrefix = new Hashtable();
 		private Hashtable hashtableElementsByName = new Hashtable();
 		private Hashtable hashtableAttributesByName = new Hashtable();
 		private ArrayList listOfXsdFilename = new ArrayList();
 		private Hashtable hashtableTabPageByFilename = new Hashtable();
 		private XSDObject firstElement = null;
-		private string currentXsdfileName = "";
 		private List<string> loadError = new List<string>();
 		private string originalTitle = "";
 		private DiagramBase contextualMenuPointedElement = null;
+		bool isRunningOnMono = IsRunningOnMono();
 
 		public MainForm()
 		{
 			InitializeComponent();
 
+			this.toolsToolStripMenuItem.Visible = !isRunningOnMono;
+			this.printDialog.UseEXDialog = !isRunningOnMono;
+
 			this.originalTitle = Text;
 
 			this.toolStripComboBoxSchemaElement.Sorted = true;
@@ -48,6 +50,8 @@ namespace XSDDiagram
 			this.panelDiagram.DiagramControl.ContextMenuStrip = this.contextMenuStripDiagram;
 			this.panelDiagram.DiagramControl.MouseWheel += new MouseEventHandler(DiagramControl_MouseWheel);
 			this.panelDiagram.DiagramControl.MouseClick += new MouseEventHandler(DiagramControl_MouseClick);
+			this.panelDiagram.DiagramControl.MouseHover += new EventHandler(DiagramControl_MouseHover);
+			this.panelDiagram.DiagramControl.MouseMove += new MouseEventHandler(DiagramControl_MouseMove);
 			this.panelDiagram.VirtualSize = new Size(0, 0);
 			this.panelDiagram.DiagramControl.Paint += new PaintEventHandler(DiagramControl_Paint);
 
@@ -57,6 +61,12 @@ namespace XSDDiagram
 			schemaSerializer.UnknownAttribute += new XmlAttributeEventHandler(schemaSerializer_UnknownAttribute);
 
 			//this.panelDiagram.DiagramControl.MouseMove += new MouseEventHandler(DiagramControl_MouseMove);
+			//toolTip.SetToolTip(panelDiagram.DiagramControl, "Dummy DiagramControl");
+		}
+
+		public static bool IsRunningOnMono()
+		{
+			return Type.GetType("Mono.Runtime") != null;
 		}
 
 		private void MainForm_Load(object sender, EventArgs e)
@@ -99,21 +109,82 @@ namespace XSDDiagram
 		private void saveDiagramToolStripMenuItem_Click(object sender, EventArgs e)
 		{
 			SaveFileDialog saveFileDialog = new SaveFileDialog();
-			saveFileDialog.Filter = "emf files (*.emf)|*.emf|All files (*.*)|*.*";
+            saveFileDialog.Filter = "SVG files (*.svg)|*.svg" + (isRunningOnMono ? "" : "|EMF files (*.emf)|*.emf") + "|PNG files (*.png)|*.png|All files (*.*)|*.*";
 			saveFileDialog.FilterIndex = 1;
 			saveFileDialog.RestoreDirectory = true;
 			if (saveFileDialog.ShowDialog() == DialogResult.OK)
 			{
-				Graphics g = this.panelDiagram.DiagramControl.CreateGraphics();
-				IntPtr hdc = g.GetHdc();
-				Metafile metafile = new Metafile(saveFileDialog.FileName, hdc);
-				g.ReleaseHdc(hdc);
-				g.Dispose();
+				try
+				{
+                    Graphics g1 = this.panelDiagram.DiagramControl.CreateGraphics();
 
-				g = Graphics.FromImage(metafile);
-				this.diagram.Layout(g);
-				this.diagram.Paint(g);
-				g.Dispose();
+					string extension = Path.GetExtension(saveFileDialog.FileName).ToLower();
+                    if (string.IsNullOrEmpty(extension)) { extension = ".svg"; saveFileDialog.FileName += extension;  }
+                    if (extension.CompareTo(".emf") == 0)
+                    {
+                        float scaleSave = this.diagram.Scale;
+                        try
+                        {
+                            this.diagram.Scale = 1.0f;
+                            this.diagram.Layout(g1);
+                            IntPtr hdc = g1.GetHdc();
+                            Metafile metafile = new Metafile(saveFileDialog.FileName, hdc);
+                            Graphics g2 = Graphics.FromImage(metafile);
+                            g2.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
+                            this.diagram.Layout(g2);
+                            this.diagram.Paint(g2);
+                            g1.ReleaseHdc(hdc);
+                            metafile.Dispose();
+                            g2.Dispose();
+                        }
+                        finally
+                        {
+                            this.diagram.Scale = scaleSave;
+                            this.diagram.Layout(g1);
+                        }
+                    }
+                    else if (extension.CompareTo(".png") == 0)
+                    {
+                        Rectangle bbox = this.diagram.ScaleRectangle(this.diagram.BoundingBox);
+                        bool bypassAlert = true;
+                        if (bbox.Width > 10000 || bbox.Height > 10000)
+                            bypassAlert = MessageBox.Show(this, string.Format("Are you agree to generate a {0}x{1} image?", bbox.Width, bbox.Height), "Huge image generation", MessageBoxButtons.YesNo) == DialogResult.Yes;
+                        if (bypassAlert)
+                        {
+                            Bitmap bitmap = new Bitmap(bbox.Width, bbox.Height);
+                            Graphics graphics = Graphics.FromImage((Image)bitmap);
+                            graphics.FillRectangle(Brushes.White, 0, 0, bbox.Width, bbox.Height);
+                            this.diagram.Paint(graphics);
+                            bitmap.Save(saveFileDialog.FileName);
+                        }
+                    }
+                    else //if (extension.CompareTo(".svg") == 0)
+					{
+                        float scaleSave = this.diagram.Scale;
+                        try
+                        {
+                            this.diagram.Scale = 1.0f;
+                            this.diagram.Layout(g1);
+                            string svgFileContent = this.diagram.ToSVG();
+                            using (StreamWriter sw = new StreamWriter(saveFileDialog.FileName))
+                            {
+                                sw.WriteLine(svgFileContent);
+                                sw.Close();
+                            }
+                        }
+                        finally
+                        {
+                            this.diagram.Scale = scaleSave;
+                            this.diagram.Layout(g1);
+                        }
+					}
+                    g1.Dispose();
+				}
+				catch (Exception ex)
+				{
+                    MessageBox.Show(ex.Message);
+					System.Diagnostics.Trace.WriteLine(ex.ToString());
+				}
 			}
 		}
 
@@ -140,9 +211,9 @@ namespace XSDDiagram
 			Rectangle clipping = new Rectangle(new Point(column * e.MarginBounds.Width, row * e.MarginBounds.Height),
 				new Size((column + 1) * e.MarginBounds.Width, (row + 1) * e.MarginBounds.Height));
 
-			e.Graphics.Clip = new Region(e.MarginBounds);
+			//MONOFIX e.Graphics.Clip = new Region(e.MarginBounds);
 
-			Point virtualPoint = this.panelDiagram.VirtualPoint;
+			//Point virtualPoint = this.panelDiagram.VirtualPoint;
 			e.Graphics.TranslateTransform(-(float)(clipping.Left - e.MarginBounds.Left), -(float)(clipping.Top - e.MarginBounds.Top));
 
 			this.diagram.Paint(e.Graphics, clipping);
@@ -203,6 +274,9 @@ namespace XSDDiagram
 		{
 			Point virtualPoint = this.panelDiagram.VirtualPoint;
 			e.Graphics.TranslateTransform(-(float)virtualPoint.X, -(float)virtualPoint.Y);
+			e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
+			e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
+
 			this.diagram.Paint(e.Graphics, new Rectangle(virtualPoint, this.panelDiagram.DiagramControl.ClientRectangle.Size));
 		}
 
@@ -232,7 +306,6 @@ namespace XSDDiagram
 		{
 			Cursor = Cursors.WaitCursor;
 
-			this.currentXsdfileName = fileName;
 			UpdateTitle(fileName);
 
 			this.diagram.Clear();
@@ -303,8 +376,6 @@ namespace XSDDiagram
 				fileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
 				this.loadError.Clear();
 				XMLSchema.schema schemaDOM = (XMLSchema.schema)schemaSerializer.Deserialize(fileStream);
-				//string prefix = Path.GetFileNameWithoutExtension(fileName);
-				//this.hashtableNamespaceUrlByPrefix[schemaDOM.targetNamespace] = prefix;
 
 				this.listOfXsdFilename.Add(fileName);
 
@@ -354,7 +425,8 @@ namespace XSDDiagram
 
 					if (!string.IsNullOrEmpty(schemaLocation))
 					{
-						loadedFileName = basePath + '\\' + schemaLocation.Replace('/', '\\');
+						loadedFileName = basePath + Path.DirectorySeparatorChar + schemaLocation.Replace('/', Path.DirectorySeparatorChar);
+
 						string url = schemaLocation.Trim();
 						if (url.IndexOf("http://") == 0 || url.IndexOf("https://") == 0)
 						{
@@ -362,7 +434,7 @@ namespace XSDDiagram
 							if (uri.Segments.Length > 0)
 							{
 								string fileNameToImport = uri.Segments[uri.Segments.Length - 1];
-								loadedFileName = basePath + '\\' + fileNameToImport;
+								loadedFileName = basePath + Path.DirectorySeparatorChar + fileNameToImport;
 								if (!File.Exists(loadedFileName))
 								{
 									WebClient webClient = new WebClient();
@@ -584,7 +656,7 @@ namespace XSDDiagram
 					else if (element.type != null)
 					{
 						XSDObject xsdObject = this.hashtableElementsByName[QualifiedNameToFullName("type", element.type)] as XSDObject;
-						if(xsdObject != null)
+						if (xsdObject != null)
 						{
 							XMLSchema.annotated annotatedElement = xsdObject.Tag as XMLSchema.annotated;
 							if (annotatedElement is XMLSchema.complexType)
@@ -615,8 +687,7 @@ namespace XSDDiagram
 
 				this.listViewAttributes.Items.Clear();
 				foreach (XSDAttribute attribute in listAttributes)
-					this.listViewAttributes.Items.Add(new ListViewItem(new string[]
-							{ attribute.Name, attribute.Type, attribute.Use, attribute.DefaultValue })).Tag = attribute;
+					this.listViewAttributes.Items.Add(new ListViewItem(new string[] { attribute.Name, attribute.Type, attribute.Use, attribute.DefaultValue })).Tag = attribute;
 			}
 		}
 
@@ -655,12 +726,12 @@ namespace XSDDiagram
 						case XMLSchema.ItemsChoiceType4.simpleContent:
 						case XMLSchema.ItemsChoiceType4.complexContent:
 							XMLSchema.annotated annotatedContent = null;
-							if(complexType.Items[i] is XMLSchema.complexContent)
+							if (complexType.Items[i] is XMLSchema.complexContent)
 							{
 								XMLSchema.complexContent complexContent = complexType.Items[i] as XMLSchema.complexContent;
 								annotatedContent = complexContent.Item;
 							}
-							else if(complexType.Items[i] is XMLSchema.simpleContent)
+							else if (complexType.Items[i] is XMLSchema.simpleContent)
 							{
 								XMLSchema.simpleContent simpleContent = complexType.Items[i] as XMLSchema.simpleContent;
 								annotatedContent = simpleContent.Item;
@@ -1041,13 +1112,13 @@ namespace XSDDiagram
 			{
 				if (e.Delta > 0)
 				{
-					if (this.toolStripComboBoxZoom.SelectedIndex > 0)
-						this.toolStripComboBoxZoom.SelectedIndex--;
+					if (this.toolStripComboBoxZoom.SelectedIndex < this.toolStripComboBoxZoom.Items.Count - 1)
+						this.toolStripComboBoxZoom.SelectedIndex++;
 				}
 				else
 				{
-					if (this.toolStripComboBoxZoom.SelectedIndex < this.toolStripComboBoxZoom.Items.Count - 1)
-						this.toolStripComboBoxZoom.SelectedIndex++;
+					if (this.toolStripComboBoxZoom.SelectedIndex > 0)
+						this.toolStripComboBoxZoom.SelectedIndex--;
 				}
 			}
 		}
@@ -1080,7 +1151,10 @@ namespace XSDDiagram
 		{
 			try
 			{
-				this.printDialog.ShowDialog(this);
+				if (this.printDialog.ShowDialog(this) == DialogResult.OK)
+				{
+					printDocument.Print();
+				}
 			}
 			catch (Exception ex)
 			{
@@ -1151,8 +1225,10 @@ namespace XSDDiagram
 				WebBrowser webBrowser = tabControlView.SelectedTab.Controls[0] as WebBrowser;
 				if (webBrowser != null)
 				{
-					if(webBrowser.Url == null || webBrowser.Url != new Uri(tabControlView.SelectedTab.Tag as string))
-						webBrowser.Navigate(tabControlView.SelectedTab.Tag as string);
+					string url = tabControlView.SelectedTab.Tag as string;
+					//if (webBrowser.Url == null || webBrowser.Url != new Uri(url))
+					if(webBrowser.Document == null)
+						webBrowser.Navigate(url);
 					webBrowser.Select();
 				}
 			}
@@ -1204,7 +1280,7 @@ namespace XSDDiagram
 				case "Center": this.diagram.Alignement = DiagramBase.Alignement.Center; break;
 				case "Bottom": this.diagram.Alignement = DiagramBase.Alignement.Far; break;
 			}
-		    UpdateDiagram();
+			UpdateDiagram();
 		}
 
 		void diagram_RequestAnyElement(DiagramBase diagramElement, out XMLSchema.element element, out string nameSpace)
@@ -1240,18 +1316,22 @@ namespace XSDDiagram
 				foreach (ListViewItem lvi in this.listViewElements.SelectedItems)
 				{
 					XSDObject xsdObject = lvi.Tag as XSDObject;
-					switch (xsdObject.Type)
-					{
-						case "element":
-							this.diagram.AddElement(xsdObject.Tag as XMLSchema.element, xsdObject.NameSpace);
-							break;
-						case "group":
-							this.diagram.AddCompositors(xsdObject.Tag as XMLSchema.group, xsdObject.NameSpace);
-							break;
-						case "complexType":
-							this.diagram.AddComplexType(xsdObject.Tag as XMLSchema.complexType, xsdObject.NameSpace);
-							break;
-					}
+					this.diagram.Add(xsdObject.Tag as XMLSchema.openAttrs, xsdObject.NameSpace);
+					//switch (xsdObject.Type)
+					//{
+					//    case "element":
+					//        this.diagram.AddElement(xsdObject.Tag as XMLSchema.element, xsdObject.NameSpace);
+					//        break;
+					//    case "group":
+					//        this.diagram.AddCompositors(xsdObject.Tag as XMLSchema.group, xsdObject.NameSpace);
+					//        break;
+					//    case "complexType":
+					//        this.diagram.AddComplexType(xsdObject.Tag as XMLSchema.complexType, xsdObject.NameSpace);
+					//        break;
+					//    case "simpleType":
+					//        this.diagram.Add(xsdObject.Tag as XMLSchema.simpleType, xsdObject.NameSpace);
+					//        break;
+					//}
 				}
 				UpdateDiagram();
 			}
@@ -1451,23 +1531,69 @@ namespace XSDDiagram
 
 		void DiagramControl_MouseMove(object sender, MouseEventArgs e)
 		{
-			//System.Diagnostics.Trace.WriteLine("toolTipDiagramElement_Popup");
-			//Point contextualMenuMousePosition = this.panelDiagram.DiagramControl.PointToClient(MousePosition);
-			//contextualMenuMousePosition.Offset(this.panelDiagram.VirtualPoint);
-			//DiagramBase resultElement;
-			//DiagramBase.HitTestRegion resultRegion;
-			//this.diagram.HitTest(contextualMenuMousePosition, out resultElement, out resultRegion);
-			//if (resultRegion != DiagramBase.HitTestRegion.None)
+			//toolTip.Show("Coucou", panelDiagram.DiagramControl, 200);
+		}
+
+		void DiagramControl_MouseHover(object sender, EventArgs e)
+		{
+		}
+
+		//private void toolTip_Popup(object sender, PopupEventArgs e)
+		//{
+		//    //toolTip.SetToolTip(e.AssociatedControl, "AAAAAAAAAA");
+		//}
+
+		private void toolTip_Draw(object sender, DrawToolTipEventArgs e)
+		{
+			Point diagramMousePosition = e.AssociatedControl.PointToClient(MousePosition);
+			string text = string.Format("AAAA {0} {1}\nA Que\n\nCoucou", diagramMousePosition.X, diagramMousePosition.Y);
+
+			Size textSize = TextRenderer.MeasureText(text, e.Font);
+			Rectangle newBound = new Rectangle(e.Bounds.X + 20, e.Bounds.Y - 20, textSize.Width + 10, textSize.Height + 10);
+
+			DrawToolTipEventArgs newArgs = new DrawToolTipEventArgs(e.Graphics,
+				e.AssociatedWindow, e.AssociatedControl, newBound, text,
+				this.BackColor, this.ForeColor, e.Font);
+			newArgs.DrawBackground();
+			newArgs.DrawBorder();
+			newArgs.DrawText(TextFormatFlags.TextBoxControl);
+
+			//e.DrawBackground();
+			//e.DrawBorder();
+			//using (StringFormat sf = new StringFormat())
 			//{
-			//    if (resultRegion == DiagramBase.HitTestRegion.Element) // && resultElement.Parent == null)
+			//    sf.Alignment = StringAlignment.Center;
+			//    sf.LineAlignment = StringAlignment.Center;
+			//    sf.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.None;
+			//    sf.FormatFlags = StringFormatFlags.NoWrap;
+			//    using (Font f = new Font("Tahoma", 9))
 			//    {
-			//        //this.contextualMenuPointedElement = resultElement;
-			//        //toolTipDiagramElement.SetToolTip(this.panelDiagram.DiagramControl, "coucou");
-			//        e.Cancel = true;
-			//        toolTipElement.Show("Coucou", this);
+			//        e.Graphics.DrawString(text, f,
+			//            SystemBrushes.ActiveCaptionText, e.Bounds, sf);
 			//    }
 			//}
+			//e.DrawText();
 		}
+
+		//void DiagramControl_MouseMove(object sender, MouseEventArgs e)
+		//{
+		//System.Diagnostics.Trace.WriteLine("toolTipDiagramElement_Popup");
+		//Point contextualMenuMousePosition = this.panelDiagram.DiagramControl.PointToClient(MousePosition);
+		//contextualMenuMousePosition.Offset(this.panelDiagram.VirtualPoint);
+		//DiagramBase resultElement;
+		//DiagramBase.HitTestRegion resultRegion;
+		//this.diagram.HitTest(contextualMenuMousePosition, out resultElement, out resultRegion);
+		//if (resultRegion != DiagramBase.HitTestRegion.None)
+		//{
+		//    if (resultRegion == DiagramBase.HitTestRegion.Element) // && resultElement.Parent == null)
+		//    {
+		//        //this.contextualMenuPointedElement = resultElement;
+		//        //toolTipDiagramElement.SetToolTip(this.panelDiagram.DiagramControl, "coucou");
+		//        e.Cancel = true;
+		//        toolTipElement.Show("Coucou", this);
+		//    }
+		//}
+		//}
 	}
 
 	public class XSDObject
