@@ -1,3 +1,4 @@
+
 using System;
 using System.Collections.Generic;
 using System.Windows.Forms;
@@ -12,9 +13,15 @@ namespace XSDDiagram
 		[STAThread]
 		static void Main()
 		{
+			Application.ThreadException += HandleThreadException;
 			Application.EnableVisualStyles();
 			Application.SetCompatibleTextRenderingDefault(false);
 			Application.Run(new MainForm());
 		}
+
+		static void HandleThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
+		{
+			System.Diagnostics.Trace.WriteLine(e.ToString());
+		}
 	}
 }
\ No newline at end of file
